@@ -34,10 +34,10 @@ class FocalSet:
         else:
             self.fod = fod
         
-        # Type of bpa
+        # LIST or ARRAY ########################################################
         if isinstance(bpa,(list,np.ndarray)):
             
-            if not isinstance(fe, (list,np.ndarray)):
+            if not isinstance(fe, (list,(list, np.ndarray))):
                 raise ValueError("If bpa is a list, the set of focal element must be algo given using 'fe'")
             
             # Check that there are not repeated focal elements
@@ -51,41 +51,57 @@ class FocalSet:
             
             # TODO if bpa are strings then check if are subsets and translate
             # Check if they are strings, and if they are pass them to indexes
-            if np.issubdtype(fe.dtype, np.str_):
-                print("To integer...")
-                to_integer = lambda x: fod.get_index(x)
-                vfunc = np.vectorize(to_integer)
-                fe = vfunc(fe)
-                # TODO if fe are integeres then are Indexes
-            elif np.issubdtype(fe.dtype, np.integer):
-                check_integer = lambda x: x < 2**fod.n and x > 0 
-                vfunc = np.vectorize(check_integer)
-                vfunc(fe) #TODO try catch
-            else:
-                raise ValueError("The values in fe must be strings representing subsets or integers representing indexes")
+            
+            if isinstance(fe, (np.ndarray)):
+                if np.issubdtype(fe.dtype, np.str_):
+                    to_integer = lambda x: fod.get_index(x)
+                    vfunc = np.vectorize(to_integer)
+                    fe = vfunc(fe)
+                    # TODO if fe are integeres then are Indexes
+                elif np.issubdtype(fe.dtype, np.integer):
+                    check_integer = lambda x: x < 2**fod.n and x > 0 
+                    vfunc = np.vectorize(check_integer)
+                    vfunc(fe) #TODO try catch
+                else:
+                    raise ValueError("The values in fe must be strings representing subsets or integers representing indexes")
+            else: # list because I checked this before
+                if isinstance(fe[0], str):
+                    to_integer = lambda x: fod.get_index(x)
+                    vfunc = np.vectorize(to_integer)
+                    fe = vfunc(fe)
+                    # TODO if fe are integeres then are Indexes
+                elif isinstance(fe[0], int):
+                    check_integer = lambda x: x < 2**fod.n and x > 0 
+                    vfunc = np.vectorize(check_integer)
+                    vfunc(fe) #TODO try catch
+                else:
+                    raise ValueError("The values in fe must be strings representing subsets or integers representing indexes")
             # Check if they are integers, they must be on the range [0,2**n]
             
             # Create the dictionary
             self.bpas = dict(zip(fe, bpa))
-            
+        
+        # DICTIONARY ###########################################################
         elif isinstance(bpa,(dict)):
-            # Ensure that the sum of the probabilities is 1
-            self.bpas = bpa
+            # TODO: Ensure that the sum of the probabilities is 1            
             
             # TODO if bpa are strings then check if are subsets and translate
             # Check if they are strings, and if they are pass them to indexes
-            # if np.issubdtype(bpa.dtype, np.str_):
-            #     print("To integer...")
-            #     to_integer = lambda x: fod.get_index(x)
-            #     vfunc = np.vectorize(to_integer)
-            #     fe = vfunc(fe)
-            #     # TODO if fe are integeres then are Indexes
-            # elif np.issubdtype(fe.dtype, np.integer):
-            #     check_integer = lambda x: x < 2**fod.n and x > 0 
-            #     vfunc = np.vectorize(check_integer)
-            #     vfunc(fe) #TODO try catch
-            # else:
-            #     raise ValueError("The values in fe must be strings representing subsets or integers representing indexes")
+            
+            if isinstance(list(bpa.keys())[0], (str)):
+                to_integer = lambda x: fod.get_index(x)
+                vfunc = np.vectorize(to_integer)
+                fe = vfunc(list(bpa.keys()))
+                
+                self.bpas = dict(zip(fe, list(bpa.values())))
+                # TODO if fe are integeres then are Indexes
+            elif isinstance(list(bpa.keys())[0], (int)):
+                check_integer = lambda x: x < 2**fod.n and x > 0 
+                vfunc = np.vectorize(check_integer)
+                vfunc(list(bpa.keys())) #TODO try catch
+                self.bpas = bpa
+            else:
+                raise ValueError("The values of the elements must be strings representing subsets or integers representing indexes")
 
         else:
             raise TypeError("Invalid type, bpa must be a list or a dictionary")
@@ -122,5 +138,13 @@ class FocalSet:
     def __repr__(self):
         return "Items {} \nFocal set {} \n".format(self.fod.items, self.bpas)
         
-    def __str__(self):
-        return "Items {} \nFocal set {} \n".format(self.fod.items, self.bpas)
\ No newline at end of file
+    def __str__(self, see_subsets = False):
+        if see_subsets:
+            pelements = self.bpas.keys() 
+            pbpas = self.bpas.values()
+            to_integer = lambda x: self.fod.get_subset(x)
+            vfunc = np.vectorize(pelements)
+            # TODO trans
+            return "Items {} \nFocal set {} \n".format(self.fod.items, pbpas)
+        else:
+            return "Items {} \nFocal set {} \n".format(self.fod.items, self.bpas)
\ No newline at end of file
